==> topia-persistence-2.12.1-20170925.081514-2-sources.jar: org/nuiton/topia/generator/BinderHelperTransformer.java <==
/*
 * #%L
 * ToPIA :: Persistence
 * $Id$
 * $HeadURL$
 * %%
 * Copyright (C) 2004 - 2014 CodeLutin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */

package org.nuiton.topia.generator;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.nuiton.eugene.GeneratorUtil;
import org.nuiton.eugene.java.ObjectModelTransformerToJava;
import org.nuiton.eugene.models.object.ObjectModel;
import org.nuiton.eugene.models.object.ObjectModelAttribute;
import org.nuiton.eugene.models.object.ObjectModelClass;
import org.nuiton.eugene.models.object.ObjectModelJavaModifier;
import org.nuiton.eugene.models.object.ObjectModelOperation;
import org.nuiton.topia.persistence.TopiaEntity;
import org.nuiton.topia.persistence.util.TopiaEntityBinder;
import org.nuiton.topia.persistence.util.TopiaEntityHelper;
import org.nuiton.util.beans.BinderFactory;
import org.nuiton.util.beans.BinderModelBuilder;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;





/**
 * A template to generate a helper for {@link TopiaEntityBinder}.
 * 
 * @author tchemit &lt;chemit@codelutin.com&gt;
 * @version $Id$
 * @plexus.component role="org.nuiton.eugene.Template" role-hint="org.nuiton.topia.generator.BinderHelperTransformer"
 * @since 2.3.1
 */
public class BinderHelperTransformer extends ObjectModelTransformerToJava {

    private static final Log log =
            LogFactory.getLog(BinderHelperTransformer.class);


    @Override
    public void transformFromModel(ObjectModel model) {
        ObjectModelClass resultClass;

        List<ObjectModelClass> classes = TopiaGeneratorUtil.getEntityClasses(model, true);

        if (CollectionUtils.isEmpty(classes)) {

            // no entity classes, so no generation
            log.warn("No entity to generate, " + getClass().getName() + " is skipped");
            return;
        }
        
        String packageName = getDefaultPackageName();
        String modelName = model.getName();
        String binderHelperClazzName = modelName + "BinderHelper";
        String daoHelperClazzName = modelName + "DAOHelper";

        resultClass = createClass(binderHelperClazzName, packageName);

        setSuperClass(resultClass, BinderFactory.class);


        addImport(resultClass, TopiaEntityBinder.class);
        addImport(resultClass, TopiaEntityHelper.class);
        addImport(resultClass, TopiaEntity.class);
        addImport(resultClass, BinderModelBuilder.class);

        ObjectModelOperation op;

        op = addOperation(resultClass,
                          "getTopiaBinder",
                          "<E extends TopiaEntity> TopiaEntityBinder<E>",
                          ObjectModelJavaModifier.PUBLIC,
                          ObjectModelJavaModifier.STATIC);
        addParameter(op, "Class<E>", "entityClass");
        addParameter(op, "String", "contextName");
        setOperationBody(op, ""
+"\n"
+"        return (TopiaEntityBinder<E>) newBinder(entityClass, entityClass, contextName, TopiaEntityBinder.class);\n"
+"    "
        );

        op = addOperation(resultClass,
                          "getSimpleTopiaBinder",
                          "<E extends TopiaEntity> TopiaEntityBinder<E>",
                          ObjectModelJavaModifier.PUBLIC,
                          ObjectModelJavaModifier.STATIC);
        addParameter(op, "Class<E>", "entityClass");
        setOperationBody(op, ""
+"\n"
+"        return getTopiaBinder(entityClass, \""+modelName+"\");\n"
+"    "
        );

        op = addOperation(resultClass,
                          "registerTopiaBinder",
                          "void",
                          ObjectModelJavaModifier.PUBLIC,
                          ObjectModelJavaModifier.STATIC);
        addParameter(op, "BinderModelBuilder", "builder");
        addParameter(op, "String", "contextName");
        setOperationBody(op, ""
+"\n"
+"       registerBinderModel(builder, contextName);\n"
+"    "
        );

        op = addOperation(resultClass,
                          "registerTopiaBinder",
                          "<E extends TopiaEntity> TopiaEntityBinder<E>",
                          ObjectModelJavaModifier.PUBLIC,
                          ObjectModelJavaModifier.STATIC);
        addParameter(op, "Class<E>", "entityClass");
        addParameter(op, "BinderModelBuilder", "builder");
        addParameter(op, "String", "contextName");
        setOperationBody(op, ""
+"\n"
+"       registerBinderModel(builder, contextName);\n"
+"       return getTopiaBinder(entityClass, contextName);\n"
+"    "
        );

        op = addOperation(resultClass,
                          "copy",
                          "<E extends TopiaEntity> void",
                          ObjectModelJavaModifier.PUBLIC,
                          ObjectModelJavaModifier.STATIC);
        addParameter(op, "String", "contextName");
        addParameter(op, "E", "source");
        addParameter(op, "E", "target");
        addParameter(op, "boolean", "tech");
        setOperationBody(op, ""
+"\n"
+"        Class<E> entityClass = (Class<E>) TopiaEntityHelper.getContractClass("+daoHelperClazzName+".getContracts(), target.getClass());\n"
+"        TopiaEntityBinder<E> binder = getTopiaBinder(entityClass, contextName);\n"
+"        if (binder == null) {\n"
+"            throw new NullPointerException(\"could not find a simple topia binder of type : \" + target.getClass());\n"
+"        }\n"
+"        binder.load(source, target, tech);\n"
+"    "
        );

        op = addOperation(resultClass,
                          "simpleCopy",
                          "<E extends TopiaEntity> void",
                          ObjectModelJavaModifier.PUBLIC,
                          ObjectModelJavaModifier.STATIC);
        addParameter(op, "E", "source");
        addParameter(op, "E", "target");
        addParameter(op, "boolean", "tech");
        setOperationBody(op, ""
+"\n"
+"        Class<E> entityClass = (Class<E>) TopiaEntityHelper.getContractClass("+daoHelperClazzName+".getContracts(), target.getClass());\n"
+"        TopiaEntityBinder<E> binder = getSimpleTopiaBinder(entityClass);\n"
+"        if (binder == null) {\n"
+"            throw new NullPointerException(\"could not find a simple topia binder of type : \" + target.getClass());\n"
+"        }\n"
+"        binder.load(source, target, tech);\n"
+"    "
        );

        StringBuilder initCode = new StringBuilder();

        for (ObjectModelClass clazz : classes) {

            String prefix = getConstantPrefix(clazz);

            if (StringUtils.isEmpty(prefix)) {

                // no specific prefix, so no prefix
                if (log.isWarnEnabled()) {
                    log.warn("[" + clazz.getName() + "] Will generate constants with NO prefix, not a good idea...");
                }
            }

            setConstantPrefix(prefix);

            generateBinder(modelName, clazz, resultClass, initCode);

        }

        op = addOperation(resultClass, "initBinders", "void", ObjectModelJavaModifier.PROTECTED, ObjectModelJavaModifier.STATIC);
        setOperationBody(op, initCode.toString());

        op = addOperation(resultClass, null, (String) null, ObjectModelJavaModifier.STATIC);
        setOperationBody(op, ""
 +"\n"
+"    initBinders();\n"
+""
        );
    }

    protected void generateBinder(String modelName,
                                  ObjectModelClass clazz,
                                  ObjectModelClass resultClass,
                                  StringBuilder initCode) {

        List<ObjectModelAttribute> list = new ArrayList<ObjectModelAttribute>();

        Collection<ObjectModelAttribute> attributes = new HashSet<ObjectModelAttribute>();
        attributes.addAll(clazz.getAllOtherAttributes());
        attributes.addAll(clazz.getAttributes());

        for (ObjectModelAttribute attr : attributes) {
            if (!attr.isNavigable()) {
                continue;
            }

            if (GeneratorUtil.isNMultiplicity(attr)) {
                // not dealing with association
                continue;
            }

            list.add(attr);
        }

        String clazzName = clazz.getName();

        if (log.isDebugEnabled()) {
            log.debug("generate simple binder for " + clazzName);
        }
        addImport(resultClass, clazz);
        initCode.append(""
+"\n"
+"        BinderModelBuilder<"+clazzName+", "+clazzName+"> builder"+clazzName+" =\n"
+"            BinderModelBuilder.newEmptyBuilder("+clazzName+".class);\n"
+""
        );
        if (!list.isEmpty()) {
            initCode.append(""
+"\n"
+"        builder"+clazzName+".addSimpleProperties(\n"
+""
            );
            Iterator<ObjectModelAttribute> itr = list.iterator();
            while (itr.hasNext()) {
                ObjectModelAttribute attr = itr.next();
                String attrName = attr.getName();
                boolean hasNext = itr.hasNext();
                initCode.append(""
+"            "+clazzName+"."+getConstantName(attrName)+""+(hasNext?",\n":"")+""
                );
            }
            initCode.append(""
+"\n"
+"        );\n"
+"    "
            );
        }

        initCode.append(""
+"\n"
+"        registerTopiaBinder(builder"+clazzName+", \""+modelName+"\");\n"
+"    "
        );
    }
}

==> topia-persistence-2.14-sources.jar: org/nuiton/topia/generator/BinderHelperTransformer.java <==
/*
 * #%L
 * ToPIA :: Persistence
 * $Id$
 * $HeadURL$
 * %%
 * Copyright (C) 2004 - 2014 CodeLutin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */

package org.nuiton.topia.generator;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.nuiton.eugene.GeneratorUtil;
import org.nuiton.eugene.java.ObjectModelTransformerToJava;
import org.nuiton.eugene.models.object.ObjectModel;
import org.nuiton.eugene.models.object.ObjectModelAttribute;
import org.nuiton.eugene.models.object.ObjectModelClass;
import org.nuiton.eugene.models.object.ObjectModelJavaModifier;
import org.nuiton.eugene.models.object.ObjectModelOperation;
import org.nuiton.topia.persistence.TopiaEntity;
import org.nuiton.topia.persistence.util.TopiaEntityBinder;
import org.nuiton.topia.persistence.util.TopiaEntityHelper;
import org.nuiton.util.beans.BinderFactory;
import org.nuiton.util.beans.BinderModelBuilder;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;


/*{generator option: parentheses = false}*/
/*{generator option: writeString = +}*/

/**
 * A template to generate a helper for {@link TopiaEntityBinder}.
 * 
 * @author tchemit &lt;chemit@codelutin.com&gt;
 * @version $Id$
 * @plexus.component role="org.nuiton.eugene.Template" role-hint="org.nuiton.topia.generator.BinderHelperTransformer"
 * @since 2.3.1
 */
public class BinderHelperTransformer extends ObjectModelTransformerToJava {

    private static final Log log =
            LogFactory.getLog(BinderHelperTransformer.class);


    @Override
    public void transformFromModel(ObjectModel model) {
        ObjectModelClass resultClass;

        List<ObjectModelClass> classes = TopiaGeneratorUtil.getEntityClasses(model, true);

        if (CollectionUtils.isEmpty(classes)) {

            // no entity classes, so no generation
            log.warn("No entity to generate, " + getClass().getName() + " is skipped");
            return;
        }
        
        String packageName = getDefaultPackageName();
        String modelName = model.getName();
        String binderHelperClazzName = modelName + "BinderHelper";
        String daoHelperClazzName = modelName + "DAOHelper";

        resultClass = createClass(binderHelperClazzName, packageName);

        setSuperClass(resultClass, BinderFactory.class);


        addImport(resultClass, TopiaEntityBinder.class);
        addImport(resultClass, TopiaEntityHelper.class);
        addImport(resultClass, TopiaEntity.class);
        addImport(resultClass, BinderModelBuilder.class);

        ObjectModelOperation op;

        op = addOperation(resultClass,
                          "getTopiaBinder",
                          "<E extends TopiaEntity> TopiaEntityBinder<E>",
                          ObjectModelJavaModifier.PUBLIC,
                          ObjectModelJavaModifier.STATIC);
        addParameter(op, "Class<E>", "entityClass");
        addParameter(op, "String", "contextName");
        setOperationBody(op, ""
/*{
        return (TopiaEntityBinder<E>) newBinder(entityClass, entityClass, contextName, TopiaEntityBinder.class);
    }*/
        );

        op = addOperation(resultClass,
                          "getSimpleTopiaBinder",
                          "<E extends TopiaEntity> TopiaEntityBinder<E>",
                          ObjectModelJavaModifier.PUBLIC,
                          ObjectModelJavaModifier.STATIC);
        addParameter(op, "Class<E>", "entityClass");
        setOperationBody(op, ""
/*{
        return getTopiaBinder(entityClass, "<%=modelName%>");
    }*/
        );

        op = addOperation(resultClass,
                          "registerTopiaBinder",
                          "void",
                          ObjectModelJavaModifier.PUBLIC,
                          ObjectModelJavaModifier.STATIC);
        addParameter(op, "BinderModelBuilder", "builder");
        addParameter(op, "String", "contextName");
        setOperationBody(op, ""
/*{
       registerBinderModel(builder, contextName);
    }*/
        );

        op = addOperation(resultClass,
                          "registerTopiaBinder",
                          "<E extends TopiaEntity> TopiaEntityBinder<E>",
                          ObjectModelJavaModifier.PUBLIC,
                          ObjectModelJavaModifier.STATIC);
        addParameter(op, "Class<E>", "entityClass");
        addParameter(op, "BinderModelBuilder", "builder");
        addParameter(op, "String", "contextName");
        setOperationBody(op, ""
/*{
       registerBinderModel(builder, contextName);
       return getTopiaBinder(entityClass, contextName);
    }*/
        );

        op = addOperation(resultClass,
                          "copy",
                          "<E extends TopiaEntity> void",
                          ObjectModelJavaModifier.PUBLIC,
                          ObjectModelJavaModifier.STATIC);
        addParameter(op, "String", "contextName");
        addParameter(op, "E", "source");
        addParameter(op, "E", "target");
        addParameter(op, "boolean", "tech");
        setOperationBody(op, ""
/*{
        Class<E> entityClass = (Class<E>) TopiaEntityHelper.getContractClass(<%=daoHelperClazzName%>.getContracts(), target.getClass());
        TopiaEntityBinder<E> binder = getTopiaBinder(entityClass, contextName);
        if (binder == null) {
            throw new NullPointerException("could not find a simple topia binder of type : " + target.getClass());
        }
        binder.load(source, target, tech);
    }*/
        );

        op = addOperation(resultClass,
                          "simpleCopy",
                          "<E extends TopiaEntity> void",
                          ObjectModelJavaModifier.PUBLIC,
                          ObjectModelJavaModifier.STATIC);
        addParameter(op, "E", "source");
        addParameter(op, "E", "target");
        addParameter(op, "boolean", "tech");
        setOperationBody(op, ""
/*{
        Class<E> entityClass = (Class<E>) TopiaEntityHelper.getContractClass(<%=daoHelperClazzName%>.getContracts(), target.getClass());
        TopiaEntityBinder<E> binder = getSimpleTopiaBinder(entityClass);
        if (binder == null) {
            throw new NullPointerException("could not find a simple topia binder of type : " + target.getClass());
        }
        binder.load(source, target, tech);
    }*/
        );

        StringBuilder initCode = new StringBuilder();

        for (ObjectModelClass clazz : classes) {

            String prefix = getConstantPrefix(clazz);

            if (StringUtils.isEmpty(prefix)) {

                // no specific prefix, so no prefix
                if (log.isWarnEnabled()) {
                    log.warn("[" + clazz.getName() + "] Will generate constants with NO prefix, not a good idea...");
                }
            }

            setConstantPrefix(prefix);

            generateBinder(modelName, clazz, resultClass, initCode);

        }

        op = addOperation(resultClass, "initBinders", "void", ObjectModelJavaModifier.PROTECTED, ObjectModelJavaModifier.STATIC);
        setOperationBody(op, initCode.toString());

        op = addOperation(resultClass, null, (String) null, ObjectModelJavaModifier.STATIC);
        setOperationBody(op, ""
 /*{
    initBinders();
}*/
        );
    }

    protected void generateBinder(String modelName,
                                  ObjectModelClass clazz,
                                  ObjectModelClass resultClass,
                                  StringBuilder initCode) {

        List<ObjectModelAttribute> list = new ArrayList<ObjectModelAttribute>();

        Collection<ObjectModelAttribute> attributes = new HashSet<ObjectModelAttribute>();
        attributes.addAll(clazz.getAllOtherAttributes());
        attributes.addAll(clazz.getAttributes());

        for (ObjectModelAttribute attr : attributes) {
            if (!attr.isNavigable()) {
                continue;
            }

            if (GeneratorUtil.isNMultiplicity(attr)) {
                // not dealing with association
                continue;
            }

            list.add(attr);
        }

        String clazzName = clazz.getName();

        if (log.isDebugEnabled()) {
            log.debug("generate simple binder for " + clazzName);
        }
        addImport(resultClass, clazz);
        initCode.append(""
/*{
        BinderModelBuilder<<%=clazzName%>, <%=clazzName%>> builder<%=clazzName%> =
            BinderModelBuilder.newEmptyBuilder(<%=clazzName%>.class);
}*/
        );
        if (!list.isEmpty()) {
            initCode.append(""
/*{
        builder<%=clazzName%>.addSimpleProperties(
}*/
            );
            Iterator<ObjectModelAttribute> itr = list.iterator();
            while (itr.hasNext()) {
                ObjectModelAttribute attr = itr.next();
                String attrName = attr.getName();
                boolean hasNext = itr.hasNext();
                initCode.append(""
/*{            <%=clazzName%>.<%=getConstantName(attrName)%><%=(hasNext?",\n":"")%>}*/
                );
            }
            initCode.append(""
/*{
        );
    }*/
            );
        }

        initCode.append(""
/*{
        registerTopiaBinder(builder<%=clazzName%>, "<%=modelName%>");
    }*/
        );
    }
}
